/*
  3Com ADSL Modem USB config program
  Copyright (C) 2002 Josep Comas

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
  Author     : Josep Comas <jcomas@gna.es>
  Creation   : 12/2/2003
 
  Description: This program configure /etc/3cp4218 file.
*/


#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <regex.h>
#include <locale.h>
#include <libintl.h>
#include <libgen.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <sys/types.h>
#include <sys/stat.h>

/* config files */
#define CONFIG_FILE "/etc/3cp4218"
#define DNS_FILE "/etc/resolv.conf"

/* glade file */
#define GLADE_FILE "3cconfig.glade"

/* translation files */
#define TF_CODE "3cconfig"
#define TF_GLADE GLADE_FILE


GladeXML *xml;  /* variable used when you load glade file (main window) */
GladeXML *xmld;  /* variable used when you load glade file (dialog windows) */


/* return filename with absolute path of current program */
char *get_absolute_appname(char *target, char *filename)
{
  char path[PATH_MAX+1];  /* absolute path */
  char absolutename[PATH_MAX+NAME_MAX+2];  /* absolute path + filename */

  if (readlink("/proc/self/exe", absolutename, sizeof(absolutename)) == -1)
    return NULL;
  strcpy(path, dirname(absolutename));
  if (strcmp(path, "/") == 0)
    snprintf(absolutename, sizeof(absolutename), "/%s", filename);
  else
    snprintf(absolutename, sizeof(absolutename), "%s/%s", path, filename);
   strcpy(target, absolutename);
  return target;
}


/* convert a Internet address from a.b.c.d format to one numeric value */ 
/* if you preferer you can use inet_aton */
int addr_char_to_num(char *saddr, int *naddr)
{
  char *pbuf, *first;  /* string pointers */
  int a, b, c, d;  /* each part of an Internet address (a.b.c.d) */

  if (saddr == NULL || naddr == NULL)
    return -1;

  first = saddr;
  pbuf = strchr(first, '.');
  if (!pbuf)
    return -1;
  a=atoi(first);
  if (a < 0 || a > 255)
    return -1;

  first = pbuf+1;
  pbuf = strchr(first, '.');
  if (!pbuf)
    return -1;
  b=atoi(first);
  if (b < 0 || b > 255)
    return -1;

  first = pbuf+1;
  pbuf = strchr(first, '.');
  if (!pbuf)
    return -1;
  c=atoi(first);
  if (c < 0 || c > 255)
    return -1;

  first = pbuf+1;
  d=atoi(first);
  if (d < 0 || d > 255)
    return -1;

  *naddr = (a*256*256*256)+(b*256*256)+(c*256)+d;

  return 0;
}


/* convert a Internet address from one numeric value to a.b.c.d format */ 
/* if you preferer you can use inet_ntoa */
int addr_num_to_char(int naddr, char *saddr)
{
  if (saddr == NULL)
    return -1;

  if (sprintf(saddr, "%i.%i.%i.%i", (naddr >> 24) & 0xff, (naddr >> 16) & 0xff, (naddr >> 8) & 0xff, naddr & 0xff) >= 7)
    return 0;
  else
    return -1;
}


/* validate an Internet address (ip, mask, gateway) */
/* if you preferer you can use inet_aton to validate address */
int validate_address(char *address) {
  regex_t fil_regex;  /* for regular expression */
  int err;  /* error code to return */

  if (address == NULL)
    return -1;

  if (regcomp(&fil_regex, "^((0|[1-9][0-9]?|1[0-9][0-9]|2[0-4][0-9]|25[0-5])\\.){3}(0|[1-9][0-9]?|1[0-9][0-9]|2[0-4][0-9]|25[0-5])$", REG_EXTENDED | REG_NOSUB)) {
    printf(dgettext(TF_CODE, "Error: I can't compile regex\n"));
    return -1;
  }
  if (regexec(&fil_regex, address, 0, NULL, 0) == 0)
    err = 0;
  else
    err = -1;

  /* free regex */
  regfree(&fil_regex);

  return err;
}


/* check if mask is valid (mask includes network number and subnet number) */
/* valid mask (in binary format) are:
     .- All 0s (00000000 00000000 00000000 00000000)
     .- All 1s (11111111 11111111 11111111 11111111)
     .- First 1s followed 0s (example: 255.255.255.0 is 11111111 11111111 11111111 00000000)
   In old RFC all 0s and all 1s are reserved
   See RFC1878 & RFC1812
 */
int is_valid_mask(int mask)
{
  int check;  /* contains a valid mask */
  int i;  /* counter */

  if (mask == 0)
    return 0;
  for (i = 0, check = 0xffffffff; i < 32; i++, check = check << 1)
    if (mask == check)
      return 0;

  return -1;
}


/* returns 0 if two Internet addresses are in same network */
int is_same_network(int addr1, int addr2, int mask)
{
  if ((addr1 & mask) == (addr2 & mask))
    return 0;
  else
    return -1;
}


/* returns 0 if is a private Internet address */
int is_private_address(int addr)
{
  if ((addr >= 0x0a000000 && addr <= 0x0affffff) || (addr >= 0xac100000 && addr <= 0xac10ffff) || (addr >= 0xc0a80000 && addr <= 0xc0a8ffff))
    return 0;
  else
    return -1;
}


/* returns 0 if source is an empty string */
int is_empty(char *source) {

  if (source == NULL)
    return -1;

  while (*source != '\0')
    if (*source++ != ' ')
      return -1;

  return 0;
}


/* remove quote from string */
char *remove_quote(char *target, char *source) {

  int copy;  /* where it begins to copy */

  if ((source == NULL) || (target == NULL))
    return NULL;

  copy = strlen(source);
  if (copy > 1)
    if (*(source+copy-1) == '"')
      copy--;
  if (*source == '"') {
    source++;
    copy--;
  }
  *target = '\0';
  strncat(target, source, copy);

  return target;
}


/* add quote to string */
char *add_quote(char *target, char *source) {
  char *pbuf;  /* pointer to target string */

  if ((source == NULL) || (target == NULL))
    return NULL;

  pbuf = target;
  *target++ = '"';
  while (*source != '\0')
    *target++ = *source++;
  *target++ = '"';
  *target = '\0';
 
  return pbuf;
}


/* delete left spaces of a string */
char *ltrim(char *target, char *source) {

  if ((source == NULL) || (target == NULL))
    return NULL;

  while ((*source != '\0') && (*source == ' '))
    source++;
  strcpy(target, source);

  return target;
}


/* delete right spaces of a string */
char *rtrim(char *target, char *source) {
  char *pbuf;  /* pointer to source string */
  char *pnoblank;  /* pointer to first no blank character */

  if ((source == NULL) || (target == NULL))
    return NULL;

  pbuf = source;
  pnoblank = source;
  while (*source != '\0') {
    if (*source != ' ')
      pnoblank = source;
    source++;
  }
  *target = '\0';
  if ((*pnoblank != ' ') && (*pnoblank != '\0'))
    strncat(target, pbuf, pnoblank-pbuf+1);

  return target;
}


/* delete left and right spaces of a string */
char *alltrim(char *target, char *source) {

  if ((source == NULL) || (target == NULL))
    return NULL;

  return ltrim(target, rtrim(target, source));
}


/* get value of a variable */
char *getvalue(char *target, char *source) {
  char *pbuf;  /* pointer to target string */

  if ((source == NULL) || (target == NULL))
    return NULL;

  pbuf = target;
  while ((*source != '\0') && (*source != '\n') && (*source != '#')) 
    *target++ = *source++;
  *target = '\0';

  return pbuf;
}


/* get a value of a variable in a file 
   parameters:
     var = variable name;
     equal = type (1 -> variable = value, other value -> variable value)
     value = value to return
     occurrence = first time, second time, ... that variable appears
     filename = file where variable resides
 */
int get_variable_in_file(const char *var, int equal, char *value, int occurrence, const char *filename)
{
  char buf[256], buf2[256];  /* temp buffers */
  regex_t fil_regex;  /* for regular expression */
  FILE *source;  /* source file */
  int i = 0;  /* counter */

  
  /* check parameters */
  if (value == NULL)
    return -1;
  value[0]='\0';
  if ((var == NULL) || (strlen(var) < 1) || (occurrence < 1) || (filename == NULL))
    return -1;
  if (strpbrk(var, "# ") != NULL)
    return -1;

  /* choose regex according to variable expression (variable = value, variable value) */ 
  buf[0]='\0';
  if (equal == 1)
    snprintf(buf, sizeof(buf), "^ *%s *=.*$", var);
  else
    snprintf(buf, sizeof(buf), "^ *%s +.*$", var);
  if (regcomp(&fil_regex, buf, REG_EXTENDED | REG_NOSUB)) {
    printf(dgettext(TF_CODE, "Error: I can't compile regex\n"));
    return -1;
  }

  /* open original file */
  source = fopen(filename, "r");
  if (source == NULL) {
    if (errno == ENOENT)  /* if file doesn't exists then return empty value */
      return 0;
    else 
      printf(dgettext(TF_CODE, "Error: I can't open file\n"));
    return -1;
  }

  /* search variable and get value */
  while (fgets(buf, sizeof(buf), source) != NULL) {
    if (regexec(&fil_regex, buf, 0, NULL, 0) == 0) {
      i++;
      if (occurrence == i) {
        if (equal == 1) {  /* variable = value */
	  getvalue(buf2, strchr(buf, '=')+1);
	  alltrim(value, buf2);
	}
	else {  /* variable value */
	  getvalue(buf2, strchr(ltrim(buf2, buf), ' ')+1);
	  alltrim(value, buf2);
	}
      }
    }
  }

  /* close file */
  fclose(source);

  /* free regex */
  regfree(&fil_regex);

  return 0;
}


/* set a value of a variable in a file 
   parameters:
     var = variable name;
     equal = type (1 -> variable = value, other value -> variable value)
     value = value to return
     empty = if empty <> 1 then variable doesn't save
     occurrence = first time, second time, ... that variable appears
     filename = file where variable resides
 */
int set_variable_in_file(const char *var, int equal, char *value, int empty, int occurrence, const char *filename)
{
  char buf[256], buf2[256];  /* temp buffers */
  char tempfilename[256];  /* temp filename */
  regex_t fil_regex;  /* for regular expression */
  FILE *source, *target;  /* source and target files */
  int i = 0;  /* counter */
  int fd;  /* file descriptor for temp filename */

  
  /* check parameters */
  if ((var == NULL) || (value == NULL) || (strlen(var) < 1) || (occurrence < 1) || (filename == NULL))
    return -1;
  if (strpbrk(var, "# ") != NULL)
    return -1;

  /* choose regex according to variable expression (variable = value, variable value) */ 
  buf[0]='\0';
  if (equal == 1)
    snprintf(buf, sizeof(buf), "^ *%s *=.*$", var);
  else
    snprintf(buf, sizeof(buf), "^ *%s +.*$", var);
  if (regcomp(&fil_regex, buf, REG_EXTENDED | REG_NOSUB)) {
    printf(dgettext(TF_CODE, "Error: I can't compile regex\n"));
    return -1;
  }

  /* open original file */
  source = fopen(filename, "r");
  if (source == NULL) {
    if (errno == ENOENT) {  /* if file doesn't exists then create empty file */
      source = fopen(filename, "w+");
      if (source == NULL) {
        printf(dgettext(TF_CODE, "Error: I can't create original file\n"));
	return -1;
      }
    }
    else {
      printf(dgettext(TF_CODE, "Error: I can't open file\n"));
      return -1;
    }
  }

  /* open temp file */
  tempfilename[0]='\0';
  snprintf(tempfilename, sizeof(tempfilename), "%s.new.XXXXXX", filename);
  fd = mkstemp(tempfilename);
  if (fd == -1) {
    printf(dgettext(TF_CODE, "Error: I can't get temp filename\n"));
    return -1;
  }
  target = fdopen(fd, "w");
  if (target == NULL) {
    printf(dgettext(TF_CODE, "Error: I can't create new file\n"));
    return -1;
  }
  if (fchmod(fd, 0644) == -1) {
    printf(dgettext(TF_CODE, "Error: I can't change new file permisions\n"));
    return -1;
  }

  /* search variable and write value */
  while (fgets(buf, sizeof(buf), source) != NULL) {
    if (regexec(&fil_regex, buf, 0, NULL, 0) == 0) {
      i++;
      if (occurrence == i) {
        if (empty == 1 || ((empty != 1) && (strlen(value) > 0))) {
          fputs(var, target);
  	  if (equal == 1)  /* variable = value */
            fputc('=', target);
	  else  /* variable value */
            fputc(' ', target);
	  alltrim(buf2, value);
          fputs(buf2, target);
	  if (strchr(buf, '#') != NULL) {  /* preserve comment */
	    fputs("  ", target);
	    fputs(strchr(buf, '#'), target);
	  }
	  else
            fputc('\n', target);
	}
      }
      else
        fputs(buf, target);
	
    }
    else
      fputs(buf, target);
  }
  if (i < occurrence) {  /* not found, write variable */
    if (empty == 1 || ((empty != 1) && (strlen(value) > 0))) {
      fputs(var, target);
      if (equal == 1)
        fputc('=', target);
      else
        fputc(' ', target);
      alltrim(buf2, value);
      fputs(buf2, target);
      fputc('\n', target);
    }
  }

  /* close files */
  fclose(target);
  fclose(source);

  /* remove and rename files */
  if (remove(filename)) {
    printf(dgettext(TF_CODE, "Error: I can't remove original file\n"));
    return -1;
  }
  if (rename(tempfilename, filename)) {
    printf(dgettext(TF_CODE, "Error: I can't rename temp file\n"));
    return -1;
  }

  /* free regex */
  regfree(&fil_regex);

  return 0;
}


/* enable/disable controls according to protocol mode */
void sensitive_protocol(char *pbuf)
{
  if (pbuf != NULL)
    switch (pbuf[0]) {
      case '1':
      case '3':
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "label_path_atm_tools"), TRUE);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "entry_path_atm_tools"), TRUE);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "frame_routed"), TRUE);
        break; 
      default:
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "label_path_atm_tools"), FALSE);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "entry_path_atm_tools"), FALSE);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "frame_routed"), FALSE);
	break;
    }
}


/* create dialog */
void create_dialog(char *name)
{
  char buf[PATH_MAX+NAME_MAX+2];  /* temp buffer */

  if (get_absolute_appname(buf, GLADE_FILE) == NULL)
    return;
  xmld = glade_xml_new(buf, name);
  if (xmld == NULL)
    return;
  
  gtk_window_set_transient_for(GTK_WINDOW (glade_xml_get_widget(xmld, name)), GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")));
  glade_xml_signal_autoconnect(xmld);
}


/* display dialog with information */ 
void show_dialog_info(char *title, char *message)
{
  create_dialog("dialog_info");
  gtk_window_set_title(GTK_WINDOW (glade_xml_get_widget(xmld, "dialog_info")), title);
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xmld, "label_di")), message);
  gtk_widget_show_all(glade_xml_get_widget(xmld, "dialog_info"));
}


/* if protocol changes then enable/disable according to it */
void on_combo_entry_protocol_mode_changed(GtkEditable *editable)
{
  sensitive_protocol(gtk_editable_get_chars(editable, 0, 1));
}


/* close info dialog */
void on_button_ok_di_clicked(GtkButton *button)
{
  gtk_widget_destroy(glade_xml_get_widget(xmld, "dialog_info"));
}


/* exit without save information */
void on_button_yes_exit_clicked(GtkButton *button)
{
  gtk_exit(0);
}


/* close exit dialog (button) */
void on_button_no_exit_clicked(GtkButton *button)
{
  gtk_widget_destroy(glade_xml_get_widget(xmld, "dialog_exit"));
}


/* remove left and right spaces from fields */
void trim_fields()
{
  char buf[256];  /* temp buffer */

  gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_path_binaries")), 
    alltrim(buf, gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_path_binaries")))));
  gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_path_atm_tools")), 
    alltrim(buf, gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_path_atm_tools")))));
  gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_ip_address")), 
    alltrim(buf, gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_ip_address")))));
  gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_mask_address")), 
    alltrim(buf, gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_mask_address")))));
  gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_gateway_address")), 
    alltrim(buf, gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_gateway_address")))));
  gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver1")), 
    alltrim(buf, gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver1")))));
  gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver2")), 
    alltrim(buf, gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver2")))));
  gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver3")), 
    alltrim(buf, gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver3")))));
}


/* if ok button is clicked then save values and exit */
void on_button_ok_clicked(GtkButton *button)
{
  char *pbuf;     /* temp pointer */
  char buf[256];  /* temp buffer */
  int addr_host, addr_mask, addr_gw;


  /* validate information */

  trim_fields();

  pbuf = gtk_editable_get_chars(GTK_EDITABLE (glade_xml_get_widget(xml, "combo_entry_protocol_mode")), 0, 1);
  if (pbuf != NULL) {

    if (pbuf[0] == '1' || pbuf[0] == '3') {

      pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_ip_address")));
      if (is_empty(pbuf) == 0) {
        gtk_window_set_focus(GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")), glade_xml_get_widget(xml, "entry_ip_address"));
        show_dialog_info("Error", dgettext(TF_CODE, "You must fill IP address field!"));
        return;
      }
      if (validate_address(pbuf) != 0) {
        gtk_window_set_focus(GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")), glade_xml_get_widget(xml, "entry_ip_address"));
        show_dialog_info("Error", dgettext(TF_CODE, "Invalid IP address form!"));
        return;
      }
      addr_char_to_num(pbuf, &addr_host);
      if (is_private_address(addr_host) == 0) {
        gtk_window_set_focus(GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")), glade_xml_get_widget(xml, "entry_ip_address"));
        show_dialog_info("Error", dgettext(TF_CODE, "You are using a private IP! You must use a public IP"));
        return;
      }

      pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_mask_address")));
      if (is_empty(pbuf) == 0) {
        gtk_window_set_focus(GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")), glade_xml_get_widget(xml, "entry_mask_address"));
        show_dialog_info("Error", dgettext(TF_CODE, "You must fill mask address field!"));
        return;
      }
      if (validate_address(pbuf) != 0) {
        gtk_window_set_focus(GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")), glade_xml_get_widget(xml, "entry_mask_address"));
        show_dialog_info("Error", dgettext(TF_CODE, "Invalid mask address form!"));
        return;
      }
      addr_char_to_num(pbuf, &addr_mask);
      if (is_valid_mask(addr_mask) != 0) {
        gtk_window_set_focus(GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")), glade_xml_get_widget(xml, "entry_mask_address"));
        show_dialog_info("Error", dgettext(TF_CODE, "Mask not permitted!"));
        return;
      }

      pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_gateway_address")));
      if (is_empty(pbuf) == 0) {
        gtk_window_set_focus(GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")), glade_xml_get_widget(xml, "entry_gateway_address"));
        show_dialog_info("Error", dgettext(TF_CODE, "You must fill gateway address field!"));
        return;
      }
      if (validate_address(pbuf) != 0) {
        gtk_window_set_focus(GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")), glade_xml_get_widget(xml, "entry_gateway_address"));
        show_dialog_info("Error", dgettext(TF_CODE, "Invalid gateway address form!"));
        return;
      }
      addr_char_to_num(pbuf, &addr_gw);

      if (is_same_network(addr_host, addr_gw, addr_mask) != 0) {
        gtk_window_set_focus(GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")), glade_xml_get_widget(xml, "entry_gateway_address"));
        show_dialog_info("Error", dgettext(TF_CODE, "IP and gateway are not in same network!"));
        return;
      }

      if (addr_host == addr_gw) {
        gtk_window_set_focus(GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")), glade_xml_get_widget(xml, "entry_gateway_address"));
        show_dialog_info("Error", dgettext(TF_CODE, "IP and Gateway have same address!"));
	return;
      }
    }
  }
      
  pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver1")));
  if (is_empty(pbuf) == 0) {
    gtk_window_set_focus(GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")), glade_xml_get_widget(xml, "entry_nameserver1"));
    show_dialog_info("Error", dgettext(TF_CODE, "You must fill primary DNS address field!"));
    return;
  }
  if (validate_address(pbuf) != 0) {
    gtk_window_set_focus(GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")), glade_xml_get_widget(xml, "entry_nameserver1"));
    show_dialog_info("Error", dgettext(TF_CODE, "Invalid primary DNS address form!"));
    return;
  }
  
  pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver2")));
  if (is_empty(pbuf) != 0 && validate_address(pbuf)) {
    gtk_window_set_focus(GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")), glade_xml_get_widget(xml, "entry_nameserver2"));
    show_dialog_info("Error", dgettext(TF_CODE, "Invalid secondary DNS address form!"));
    return;
  }
  
  pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver3")));
  if (is_empty(pbuf) != 0 && validate_address(pbuf)) {
    gtk_window_set_focus(GTK_WINDOW (glade_xml_get_widget(xml, "dialog_config")), glade_xml_get_widget(xml, "entry_nameserver3"));
    show_dialog_info("Error", dgettext(TF_CODE, "Invalid tertiary DNS address form!"));
    return;
  }


  /* save information to config files */

  pbuf = gtk_editable_get_chars(GTK_EDITABLE (glade_xml_get_widget(xml, "combo_entry_protocol_mode")), 0, 1);
  if (pbuf != NULL) 
    set_variable_in_file("PROTOCOL_MODE", 1, pbuf, 1, 1, CONFIG_FILE);

  pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_path_binaries")));
  if (pbuf != NULL) {
    add_quote(buf, pbuf);
    set_variable_in_file("BINARY_PATH", 1, buf, 1, 1, CONFIG_FILE);
  }
  pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_path_atm_tools")));
  if (pbuf != NULL) {
    add_quote(buf, pbuf); 
    set_variable_in_file("ATM_PATH", 1, buf, 1, 1, CONFIG_FILE);
  }

  pbuf = gtk_editable_get_chars(GTK_EDITABLE (glade_xml_get_widget(xml, "combo_entry_open_mode")), 0, 1);
  if (pbuf != NULL) 
    set_variable_in_file("OPEN_MODE", 1, pbuf, 1, 1, CONFIG_FILE);

  snprintf(buf, sizeof(buf), "%i", (int) gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (glade_xml_get_widget(xml, "spinbutton_vpi"))));
  if (buf != NULL) 
    set_variable_in_file("VPI", 1, buf, 1, 1, CONFIG_FILE);
  snprintf(buf, sizeof(buf), "%i", (int) gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (glade_xml_get_widget(xml, "spinbutton_vci"))));
  if (buf != NULL) 
    set_variable_in_file("VCI", 1, buf, 1, 1, CONFIG_FILE);

  pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_ip_address")));
  if (pbuf != NULL) 
    set_variable_in_file("IP_ADDRESS", 1, pbuf, 1, 1, CONFIG_FILE);
  pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_mask_address")));
  if (pbuf != NULL) 
    set_variable_in_file("NETMASK", 1, pbuf, 1, 1, CONFIG_FILE);
  pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_gateway_address")));
  if (pbuf != NULL) 
    set_variable_in_file("GATEWAY", 1, pbuf, 1, 1, CONFIG_FILE);

  pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver1")));
  if (pbuf != NULL) 
    set_variable_in_file("nameserver", 0, pbuf, 0, 1, DNS_FILE);
  pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver2")));
  if (pbuf != NULL) 
    set_variable_in_file("nameserver", 0, pbuf, 0, 2, DNS_FILE);
  pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver3")));
  if (pbuf != NULL) 
    set_variable_in_file("nameserver", 0, pbuf, 0, 3, DNS_FILE);
  gtk_main_quit();
}


/* show exit dialog */
void show_dialog_exit()
{
  create_dialog("dialog_exit");
  gtk_widget_show_all(glade_xml_get_widget(xmld, "dialog_exit"));
}


/* query if exit */
void on_button_cancel_clicked(GtkButton *button)
{
  show_dialog_exit();
}


/* query if exit */
gboolean on_dialog_config_delete_event(GtkWidget *widget, GdkEvent *event, gpointer  user_data)
{
  show_dialog_exit();

  return TRUE;
}


gboolean on_entry_ip_address_focus_out_event(GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
  char *pbuf;
  char buf[16];
  int addr_host, addr_mask;

  pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_gateway_address")));
  if (is_empty(pbuf) == 0) {
    pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_ip_address")));
    if (is_empty(pbuf) != 0) {
      addr_char_to_num(pbuf, &addr_host);
      pbuf = gtk_entry_get_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_mask_address")));
      if (is_empty(pbuf) != 0) {
        addr_char_to_num(pbuf, &addr_mask);
        addr_num_to_char((addr_host & addr_mask) + 2, buf);
        gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_gateway_address")), buf);
      }
    }
  }
  
  return TRUE;
}


/* read variables from config files */
void load_configuration()
{
  char buf[256], buf2[256];
  GtkCombo *combo;
  int i, l, found;
  GList *ele;
  char *entry;

  get_variable_in_file("PROTOCOL_MODE", 1, buf, 1, CONFIG_FILE);
  if (buf != NULL) {
    combo = GTK_COMBO (glade_xml_get_widget(xml, "combo_protocol_mode"));
    if ((buf[0] == '1') || (buf[0] == '2') || (buf[0] == '3') || (buf[0] == '4')) 
      gtk_list_select_item(GTK_LIST (combo->list), buf[0]-'1');
    else
      gtk_list_select_item(GTK_LIST (combo->list), 0);
  }

  get_variable_in_file("BINARY_PATH", 1, buf2, 1, CONFIG_FILE);
  remove_quote(buf, buf2);
  if (buf != NULL) 
    gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_path_binaries")), buf);
  get_variable_in_file("ATM_PATH", 1, buf2, 1, CONFIG_FILE);
  remove_quote(buf, buf2);
  if (buf != NULL) 
    gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_path_atm_tools")), buf);

  get_variable_in_file("OPEN_MODE", 1, buf, 1, CONFIG_FILE);
  if (buf != NULL) {

    combo = GTK_COMBO (glade_xml_get_widget(xml, "combo_open_mode"));
    l = g_list_length(GTK_LIST (combo->list)->children);
    i = 0;
    found = 0;
    while (i < l && !found) {
      ele = g_list_nth(GTK_LIST (combo->list)->children, i);
      gtk_label_get(GTK_LABEL(GTK_BIN(ele->data)->child), &entry);
      if (entry != NULL)
	if (strlen(entry) >= 1)
	  if (entry[0] == buf[0]) {
            gtk_list_select_item(GTK_LIST (combo->list), i);
	    found = 1;
	  }
      i++;
    }
  }

  get_variable_in_file("VPI", 1, buf, 1, CONFIG_FILE);
  if (buf != NULL) 
    gtk_spin_button_set_value(GTK_SPIN_BUTTON (glade_xml_get_widget(xml, "spinbutton_vpi")), atof(buf));
  get_variable_in_file("VCI", 1, buf, 1, CONFIG_FILE);
  if (buf != NULL) 
    gtk_spin_button_set_value(GTK_SPIN_BUTTON (glade_xml_get_widget(xml, "spinbutton_vci")), atof(buf));

  get_variable_in_file("IP_ADDRESS", 1, buf, 1, CONFIG_FILE);
  if (buf != NULL) 
    gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_ip_address")), buf);
  get_variable_in_file("NETMASK", 1, buf, 1, CONFIG_FILE);
  if (buf != NULL) 
    gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_mask_address")), buf);
  get_variable_in_file("GATEWAY", 1, buf, 1, CONFIG_FILE);
  if (buf != NULL) 
    gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_gateway_address")), buf);

  get_variable_in_file("nameserver", 0, buf, 1, DNS_FILE);
  if (buf != NULL)
    gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver1")), buf);
  get_variable_in_file("nameserver", 0, buf, 2, DNS_FILE);
  if (buf != NULL)
    gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver2")), buf);
  get_variable_in_file("nameserver", 0, buf, 3, DNS_FILE);
  if (buf != NULL)
    gtk_entry_set_text(GTK_ENTRY (glade_xml_get_widget(xml, "entry_nameserver3")), buf);

}


int main(int argc, char *argv[]) {

  /* temp buffer */
  char buf[PATH_MAX+NAME_MAX+2];

  //setlocale(LC_ALL, "");  /* Idem gtk_set_locale, use gtk_set_locale if you use gtk functions */
  //setlocale(LC_ALL, "es_ES");  /* spanish, I prefer at shell to type "export LANG=es_ES@euro" */
  //setlocale(LC_ALL, "ca_ES");  /* catalan, I prefer at shell to type "export LANG=ca_ES@euro" */

  /* init locale */
  bindtextdomain(TF_GLADE, "/usr/share/locale");  /* set directory for a domain (glade messages) */
  bindtextdomain(TF_CODE, "/usr/share/locale");  /* set directory for a domain (source code messages) */
  textdomain(TF_GLADE);  /* set domain */
  gtk_set_locale();

  /* init gtk */
  if (gtk_init_check(&argc, &argv) == FALSE) {
    printf(dgettext(TF_CODE, "Error: GTK can't be initialized\n"));
    return -1;
  }

  /* init glade */
  glade_init();

  /* load the interface */
  if (get_absolute_appname(buf, GLADE_FILE) == NULL)
    return -1;
  xml = glade_xml_new(buf, "dialog_config");
  if (xml == NULL)
    return -1;
      
  /* read configuration */
  load_configuration();

  /* enable/disable controls according to protocol mode */
  sensitive_protocol(gtk_editable_get_chars(GTK_EDITABLE (glade_xml_get_widget(xml, "combo_entry_protocol_mode")), 0, 1));

  /* connect the signals in the interface */
  glade_xml_signal_autoconnect(xml);

  /* start the event loop */
  gtk_main();

  return 0;

}

