/*
  3Com ADSL Modem USB information panel
  Copyright (C) 2002 Josep Comas

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
  Author     : Josep Comas <jcomas@gna.es>
  Creation   : 25/8/2002
 
  Description: This program shows information about 3Com ADSL Modem USB
               (it requires kernel driver loaded).

  Log:
  
  5/6/2003 Josep Comas
  Update to work with libusb-0.1.7

  7/6/2003 Josep Comas
  Put mac address in lowercase
  Added release version for kernel module

*/


#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <locale.h>
#include <libintl.h>
#include <monetary.h>
#include <libgen.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <usb.h>
#include "usbi.h"
#include <sys/ioctl.h>
#include <sys/stat.h>

/* modem usb identifications */
#define ID_VENDOR    0x0506  /* Vendor = 3Com */
#define ID_PRODUCT1  0xF002  /* Product = ADSL modem without EZ-USB firmware loaded */
#define ID_PRODUCT1b 0xF100  /* Product = ADSL modem without EZ-USB firmware loaded */
#define ID_PRODUCT2  0xF003  /* Product = ADSL modem with EZ-USB firmware loaded */

/* command received */
#define ADSL_Operational_Mode_Ack  0x12
#define ADSL_Open_Rejected         0x21
#define ADSL_Connection_Opened     0x24
#define ADSL_Connection_Closed     0x25
#define ADSL_Connection_Lost       0x41
#define ADSL_In_Training           0x5f

/* modem status */
#define MODEM_DOWN  0x00
#define MODEM_WAIT  0x08
#define MODEM_INIT  0x10
#define MODEM_UP    0x20

/* open mode */
#define ADSL_Open            0x03
#define ADSL_Open_ANSI       0x05
#define ADSL_Open_DMT        0x06
#define ADSL_Open_GLITE      0x07
#define ADSL_Open_MULTIMODE  0x08

/* valid ADSL operational mode */
#define ADSL_OPERATIONAL_MODE_ANSI   0x01
#define ADSL_OPERATIONAL_MODE_GDMT   0x02
#define ADSL_OPERATIONAL_MODE_GLITE  0x08

/* data path */
#define FastPath             1
#define InterleavedPath      2

/* glade file */
#define GLADE_FILE "3cpanel.glade"

/* translation files */
#define TF_CODE "3cpanel"
#define TF_GLADE GLADE_FILE


GladeXML *xml = NULL;  /* variable used when you load glade file (main window) */
GladeXML *xmld = NULL;  /* variable used when you load glade file (dialog windows) */
int visibleAdvanced = 0;  /* 0 = No visible advanced tab, 1 = Visible advanced tab */

usb_dev_handle * adsl_handle;  /* used to make ioctl calls */

/* struct to call to 3Com kernel driver (module) */
struct usbdevfs_ioctl {
  int ifno;
  int ioctl_code;
  void *data;
};

/* USBDEVFS IOCTL to call to 3Com kernel driver (module) */
#define USBDEVFS_IOCTL _IOWR('U', 18, struct usbdevfs_ioctl)

struct udsl_modem_info {
  unsigned char lastcmd;
  unsigned char laststatus;
  unsigned char modminor;
  unsigned char modmajor;
  unsigned char modrelease;
  unsigned char modspeed;
  unsigned char firmminor;
  unsigned char firmmajor;
  unsigned char hardversion;
  unsigned char harddata[4];
  unsigned char mac[6];
  unsigned char serial[12];
  char alcfirmminor;
  char alcfirmmajor;
  short vpi;
  int vci;
  unsigned int atmcelltx;
  unsigned int atmcellrx;
  unsigned int oamloopback;
  unsigned int downbitrate;
  unsigned int upbitrate;
  unsigned char openmode;
  unsigned char operationalmode;
  unsigned char linestatus;
  unsigned char datapath;
  unsigned int linklost;
  unsigned char extendedinfo;
  unsigned short downcorrectederrors;
  unsigned short downuncorrectederrors;
  unsigned short downhecerrors;
  unsigned short upcorrectederrors;
  unsigned short upuncorrectederrors;
  unsigned short uphecerrors;
  unsigned char downattenuation;
  unsigned char upattenuation;
  signed char downsignalnoiseratio;
  signed char upsignalnoiseratio;
  signed char downoutputpower;
  signed char upoutputpower;
} udsl_modem_info_t;

/* IOCTL codes when call to 3Com kernel driver (module) */
#define IOCTL_START                      1
#define IOCTL_STOP                       2
#define IOCTL_INFO _IOWR('U', 3, struct udsl_modem_info)
#define IOCTL_RECONNECT                  4
#define IOCTL_UPDATE_OPS_MODE            5
#define IOCTL_START_EXTENDED_INFO        6
#define IOCTL_STOP_EXTENDED_INFO         7
#define IOCTL_CLEAR_COUNTERS             8

/* return filename with absolute path of current program */
char *get_absolute_appname(char *target, char *filename)
{
  char path[PATH_MAX+1];  /* absolute path */
  char absolutename[PATH_MAX+NAME_MAX+2];  /* absolute path + filename */

  if (readlink("/proc/self/exe", absolutename, sizeof(absolutename)) == -1)
    return NULL;
  strcpy(path, dirname(absolutename));
  if (strcmp(path, "/") == 0)
    snprintf(absolutename, sizeof(absolutename), "/%s", filename);
  else
    snprintf(absolutename, sizeof(absolutename), "%s/%s", path, filename);
  strcpy(target, absolutename);
  return target;
}	


/* check if a file exists */
int file_exists(const char *filename)
{
  struct stat info_file;

  return stat(filename, &info_file) == 0;
}


/* string to upper */
char *strtoupper(char *source)
{
  char *pbuf;

  if (source == NULL)
    return NULL;

  pbuf = source;
  while (*source != '\0')
    *source++ = toupper(*source);

  return pbuf;
}


/* create dialog */
void create_dialog(char *name)
{
  char buf[PATH_MAX+NAME_MAX+2];  /* temp buffer */

  if (get_absolute_appname(buf, GLADE_FILE) == NULL)
    return;
  xmld = glade_xml_new(buf, name);
  if (xmld == NULL)
    return;
  if (xml != NULL)
    gtk_window_set_transient_for(GTK_WINDOW (glade_xml_get_widget(xmld, name)), GTK_WINDOW (glade_xml_get_widget(xml, "dialog_panel")));
  glade_xml_signal_autoconnect(xmld);
}


/* display dialog with information */ 
void show_dialog_info(char *title, char *message)
{
  create_dialog("dialog_info");
  gtk_window_set_title(GTK_WINDOW (glade_xml_get_widget(xmld, "dialog_info")), title);
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xmld, "label_di")), message);
  gtk_widget_show_all(glade_xml_get_widget(xmld, "dialog_info"));
}


/* close info dialog (button) */
void on_button_ok_di_clicked(GtkButton *button)
{
  gtk_widget_destroy(glade_xml_get_widget(xmld, "dialog_info"));
  if (xml == NULL)
    gtk_main_quit();
}


/* close info dialog (window) */
gboolean on_dialog_info_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  if (xml == NULL)
    gtk_main_quit();
  return TRUE;
}


/* change color of a Widget */
void change_color(GtkWidget *widget, char *colorname)
{
  GdkColor color;
  GtkRcStyle *rc_style;

  gdk_color_parse(colorname, &color);

  rc_style = gtk_rc_style_new();

  rc_style->fg[GTK_WIDGET_STATE(widget)] = color;

  rc_style->color_flags[GTK_WIDGET_STATE(widget)] |= GTK_RC_FG;

  gtk_widget_modify_style(widget, rc_style);

  gtk_rc_style_unref(rc_style);

}


/* draw when needed */
gboolean on_drawingarea_led_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
  if (GTK_WIDGET_VISIBLE(widget) == TRUE)
    gdk_draw_arc(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE(widget)], TRUE, 5, 0, 21, 21, 0, 64*360);

  return TRUE;
}


/* call to kernel driver for get info about modem */
int call_driver(struct udsl_modem_info *modem_info)
{
  /* struct to call to 3Com kernel driver (module) */
  struct usbdevfs_ioctl u_ioctl;

  if (modem_info == NULL)
    return -1;

  memset(modem_info, sizeof(struct udsl_modem_info), 0);

  /* get info about modem */
  u_ioctl.ifno = 0;
  modem_info->lastcmd = 0xff;
  modem_info->laststatus = 0xff;
  u_ioctl.ioctl_code = IOCTL_INFO;
  u_ioctl.data = modem_info;

  if (ioctl(adsl_handle->fd, USBDEVFS_IOCTL, &u_ioctl) < 0)
    return -1;

  return 0;
}


void call_ioctl(int option)
{
  struct usbdevfs_ioctl u_ioctl;

  u_ioctl.ifno = 0;
  u_ioctl.ioctl_code = option;
  u_ioctl.data = NULL;

  ioctl(adsl_handle->fd, USBDEVFS_IOCTL, &u_ioctl);
}


void set_blank_advanced()
{
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_downattenuation")), "");
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_upattenuation")), "");
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_downsnr")), "");
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_upsnr")), "");
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_downpower")), "");
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_uppower")), "");
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_downcorrected")), "");
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_upcorrected")), "");
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_downuncorrected")), "");
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_upuncorrected")), "");
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_downhec")), "");
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_uphec")), "");
}


/* show data into panel */
gint get_data(gpointer data)
{
  char buf[256];  /* temp buffer */

  /* struct where put info provided by 3Com kernel driver (module) */
  struct udsl_modem_info modem_info;

  if (call_driver(&modem_info))
  {
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_module")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_hard")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_firm")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_mac")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_serial")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_vci")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_vpi")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_atmcelltx")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_atmcellrx")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_oamloopback")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_openmode")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_adsl")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_operationalmode")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_datapath")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_linklost")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_down")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_up")), "");
    set_blank_advanced();
    gtk_widget_hide_all(glade_xml_get_widget(xml, "drawingarea_led"));

    return TRUE;
  }

/*  printf("command = 0x%02x\n", modem_info.lastcmd);
  printf("status = 0x%02x\n", modem_info.laststatus);
  printf("linestatus = 0x%02x\n", modem_info.linestatus); */

  snprintf(buf, sizeof(buf), "%i.%i.%i", modem_info.modmajor, modem_info.modminor, modem_info.modrelease);
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_module")), buf);
  snprintf(buf, sizeof(buf), "%d:%02x%02x%02x%02x:%d.%d", modem_info.hardversion, modem_info.harddata[0], modem_info.harddata[1], modem_info.harddata[2], modem_info.harddata[3], modem_info.firmmajor, modem_info.firmminor);
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_hard")), buf);
  snprintf(buf, sizeof(buf), "%d.%d.%d", (int) modem_info.alcfirmmajor >> 4, (int) modem_info.alcfirmmajor &0xf, (int) modem_info.alcfirmminor);
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_firm")), buf);
  snprintf(buf, sizeof(buf), "%02x.%02x.%02x.%02x.%02x.%02x", modem_info.mac[0], modem_info.mac[1], modem_info.mac[2], modem_info.mac[3], modem_info.mac[4], modem_info.mac[5]);
  //gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_mac")), strtoupper(buf));
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_mac")), buf);
  strcpy(buf, "");
  strncat(buf, modem_info.serial, 12);
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_serial")), buf);
  snprintf(buf, sizeof(buf), "%i", modem_info.vci);
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_vci")), buf);
  snprintf(buf, sizeof(buf), "%i", modem_info.vpi);
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_vpi")), buf);
  strfmon(buf, sizeof(buf), "%!.0i", modem_info.atmcelltx / 1.0);
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_atmcelltx")), buf);
  strfmon(buf, sizeof(buf), "%!.0i", modem_info.atmcellrx / 1.0);
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_atmcellrx")), buf);
  strfmon(buf, sizeof(buf), "%!.0i", modem_info.oamloopback / 1.0);
  gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_oamloopback")), buf);

  if (modem_info.modspeed == 1) {
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_adsl")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_operationalmode")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_datapath")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_linklost")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_down")), "");
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_up")), "");
    set_blank_advanced();
    gtk_widget_hide_all(glade_xml_get_widget(xml, "drawingarea_led"));
  }
  else {
    strfmon(buf, sizeof(buf), "%!.0i", modem_info.linklost / 1.0);
    gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_linklost")), buf);

    if (modem_info.linestatus == ADSL_Connection_Opened)
    {
      gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_adsl")), dgettext(TF_CODE, "connected"));

      switch (modem_info.operationalmode)
      {
        case ADSL_OPERATIONAL_MODE_ANSI:
          gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_operationalmode")), "ANSI T1.413");
	  break;
        case ADSL_OPERATIONAL_MODE_GDMT:
          gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_operationalmode")), "ITU-T G.992.1 (G.DMT)");
	  break;
        case ADSL_OPERATIONAL_MODE_GLITE:
          gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_operationalmode")), "ITU-T G.992.2 (G.LITE)");
	  break;
        case 0xff:
          gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_operationalmode")), dgettext(TF_CODE, "not available"));
          break;
	default:
          gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_operationalmode")), dgettext(TF_CODE, "unknown"));
	  break;
      }

      switch (modem_info.datapath)
      {
	case FastPath:
          gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_datapath")), "fast");
	  break;
	case InterleavedPath:
          gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_datapath")), "interleaved");
	  break;
	default:
          gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_datapath")), dgettext(TF_CODE, "unknown"));
	  break;
      }

      strfmon(buf, sizeof(buf), "%!.0i", modem_info.downbitrate / 1.0);
      gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_down")), buf);
      strfmon(buf, sizeof(buf), "%!.0i", modem_info.upbitrate / 1.0);
      gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_up")), buf);

      change_color(glade_xml_get_widget(xml, "drawingarea_led"), "SpringGreen1");
      gtk_widget_show_all(glade_xml_get_widget(xml, "drawingarea_led"));

      if (modem_info.extendedinfo == 1) {
        snprintf(buf, sizeof(buf), "%i", modem_info.downattenuation);
        gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_downattenuation")), buf);
        snprintf(buf, sizeof(buf), "%i", modem_info.upattenuation);
        gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_upattenuation")), buf);
        snprintf(buf, sizeof(buf), "%i", modem_info.downsignalnoiseratio);
        gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_downsnr")), buf);
        snprintf(buf, sizeof(buf), "%i", modem_info.upsignalnoiseratio);
        gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_upsnr")), buf);
        snprintf(buf, sizeof(buf), "%i", modem_info.downoutputpower);
        gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_downpower")), buf);
        snprintf(buf, sizeof(buf), "%i", modem_info.upoutputpower);
        gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_uppower")), buf);

        strfmon(buf, sizeof(buf), "%!.0i", modem_info.downcorrectederrors / 1.0);
        gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_downcorrected")), buf);
        strfmon(buf, sizeof(buf), "%!.0i", modem_info.upcorrectederrors / 1.0);
        gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_upcorrected")), buf);
        strfmon(buf, sizeof(buf), "%!.0i", modem_info.downuncorrectederrors / 1.0);
        gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_downuncorrected")), buf);
        strfmon(buf, sizeof(buf), "%!.0i", modem_info.upuncorrectederrors / 1.0);
        gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_upuncorrected")), buf);
        strfmon(buf, sizeof(buf), "%!.0i", modem_info.downhecerrors / 1.0);
        gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_downhec")), buf);
        strfmon(buf, sizeof(buf), "%!.0i", modem_info.uphecerrors / 1.0);
        gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_uphec")), buf);
      }
      else
        set_blank_advanced();

    }
    else
    {
      if (modem_info.linestatus == ADSL_In_Training) {
        gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_adsl")), dgettext(TF_CODE, "in training"));
        change_color(glade_xml_get_widget(xml, "drawingarea_led"), "Yellow");
        gtk_widget_show_all(glade_xml_get_widget(xml, "drawingarea_led"));
      }
      else {
        gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_adsl")), dgettext(TF_CODE, "not connected"));
        change_color(glade_xml_get_widget(xml, "drawingarea_led"), "Red");
        gtk_widget_show_all(glade_xml_get_widget(xml, "drawingarea_led"));
      }
      gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_operationalmode")), "");
      gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_datapath")), "");
      gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_down")), "");
      gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_up")), "");
      set_blank_advanced();
    }

  }
  switch (modem_info.openmode)
  {
    case ADSL_Open_ANSI:
      gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_openmode")), "ANSI T1.413");
      break;
    case ADSL_Open_DMT:
      gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_openmode")), "ITU-T G.992.1 (G.DMT)");
      break;
    case ADSL_Open_GLITE: 
      gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_openmode")), "ITU-T G.992.2 (G.LITE)");
      break;
    case ADSL_Open_MULTIMODE: 
      gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_openmode")), dgettext(TF_CODE, "multimode"));
      break;
    case ADSL_Open:
      gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_openmode")), dgettext(TF_CODE, "other"));
      break;
    case 0xff:
      gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_openmode")), dgettext(TF_CODE, "not available"));
      break;
    default:
      gtk_label_set_text(GTK_LABEL (glade_xml_get_widget(xml, "label_openmode")), dgettext(TF_CODE, "unknown"));
      break;
  }

  return TRUE;
}


void on_button_clear_clicked(GtkButton *button)
{
  call_ioctl(IOCTL_CLEAR_COUNTERS);
}


void on_button_advanced_clicked(GtkButton *button)
{
  if (visibleAdvanced == 1) {
    call_ioctl(IOCTL_STOP_EXTENDED_INFO);
    gtk_widget_hide_all(gtk_notebook_get_nth_page(GTK_NOTEBOOK (glade_xml_get_widget(xml, "notebook1")), 3));
    visibleAdvanced = 0;
  }
  else {
    call_ioctl(IOCTL_START_EXTENDED_INFO);
    gtk_widget_show_all(gtk_notebook_get_nth_page(GTK_NOTEBOOK (glade_xml_get_widget(xml, "notebook1")), 3));
    visibleAdvanced = 1;
  }

}


/* search modem */
int check_modem()
{
  /* bus structures variables */
  struct usb_bus * bus;
  struct usb_device * dev;
  struct usb_device * adsl_dev = NULL;

  /* boolean value */
  int goon;


  /* init USB bus and find devices */
  usb_init();
  if (usb_find_busses() < 0)
  {
    show_dialog_info(dgettext(TF_CODE, "Error"), dgettext(TF_CODE, "I can't find busses"));
    gtk_main();
    return -1;
  }
  if (usb_find_devices() < 0)
  {
    show_dialog_info(dgettext(TF_CODE, "Error"), dgettext(TF_CODE, "I can't find devices"));
    gtk_main();
    return -1;
  }

  /* search first ADSL modem */
  bus = usb_busses;
  goon = 1;
  while (bus && goon)
  {
    dev = bus->devices;
    while (dev && goon)
    {
      if (dev->descriptor.idVendor == ID_VENDOR)
      {
        if (dev->descriptor.idProduct == ID_PRODUCT1 ||
            dev->descriptor.idProduct == ID_PRODUCT1b)
	{
	  show_dialog_info(dgettext(TF_CODE, "Error"), dgettext(TF_CODE, "ADSL modem isn't initialized"));
	  gtk_main();
	  return -1;
	}
        else if (dev->descriptor.idProduct == ID_PRODUCT2)
        {
          goon = 0;
          adsl_dev = dev;
        }
      }
      if (goon)
        dev = dev->next;
    }
    if (goon)
      bus = bus->next;
  }
  if (adsl_dev == NULL)
  {
    printf(dgettext(TF_CODE, "I didn't find ADSL modem"));
    show_dialog_info(dgettext(TF_CODE, "Error"), dgettext(TF_CODE, "I didn't find ADSL modem"));
    gtk_main();
    return -1;
  }

  /* connect to ADSL modem */
  adsl_handle = usb_open(adsl_dev);
  if (adsl_handle == NULL)
  {
    show_dialog_info(dgettext(TF_CODE, "Error"), dgettext(TF_CODE, "I can't get device handle for ADSL modem"));
    gtk_main();
    return -1;
  }
  if (usb_set_configuration(adsl_handle, 1) < 0)
  {
    show_dialog_info(dgettext(TF_CODE, "Error"), dgettext(TF_CODE, "I can't set ADSL modem configuration"));
    gtk_main();
    usb_close(adsl_handle);
    return -1;
  }


  return 0;
}


int main(int argc, char *argv[]) {

  /* struct where put info provided by 3Com kernel driver (module) */
  struct udsl_modem_info modem_info;

  /* temp buffer */
  char buf[PATH_MAX+NAME_MAX+2]; 


  /*setlocale(LC_ALL, "");*/  /* Idem gtk_set_locale, use gtk_set_locale if you use gtk functions */
  /*setlocale(LC_ALL, "es_ES");*/  /* spanish, I prefer at shell to type "export LANG=es_ES@euro" */
  /*setlocale(LC_ALL, "ca_ES");*/  /* catalan, I prefer at shell to type "export LANG=ca_ES@euro" */

  /* init locale */
  //if (file_exists("./locale")) {
  //  bindtextdomain(TF_GLADE, "./locale");  /* set directory for a domain (glade messages) */
  //  bindtextdomain(TF_CODE, "./locale");  /* set directory for a domain (source code messages) */
  //}
  //else {
    bindtextdomain(TF_GLADE, "/usr/share/locale");  /* set directory for a domain (glade messages) */
    bindtextdomain(TF_CODE, "/usr/share/locale");  /* set directory for a domain (source code messages) */
  //}
  textdomain(TF_GLADE);  /* set domain */
  gtk_set_locale();

  /* init gtk */
  if (gtk_init_check(&argc, &argv) == FALSE) {
    printf(dgettext(TF_CODE, "Error: GTK can't be initialized\n"));
    return -1;
  }

  /* init glade */
  glade_init();

  /* find modem */
  if (check_modem())
    return -1;

  /* check if kernel driver is loaded */
  if (call_driver(&modem_info))
  {
    show_dialog_info(dgettext(TF_CODE, "Error"), dgettext(TF_CODE, "I can't connect with ADSL modem driver"));
    gtk_main();
    usb_close(adsl_handle);
    return -1;
  }
  call_ioctl(IOCTL_STOP_EXTENDED_INFO);

  /* load the interface */
  if (get_absolute_appname(buf, GLADE_FILE) == NULL)
    return -1;
  xml = glade_xml_new(buf, "dialog_panel");
  if (xml == NULL)
    return -1;

  /* connect the signals in the interface */
  glade_xml_signal_autoconnect(xml);

  /* check periodic state (modem) */
  gtk_timeout_add(1000, &get_data, NULL);

  /* hide advanced tab */
  gtk_widget_hide_all(gtk_notebook_get_nth_page(GTK_NOTEBOOK (glade_xml_get_widget(xml, "notebook1")), 3));

  /* start the event loop */
  gtk_main();

  call_ioctl(IOCTL_STOP_EXTENDED_INFO);

  /* close usb handle */
  usb_close(adsl_handle);

  return 0;

}

